package src.BAALL

import akka.actor.ActorDSL._
import akka.actor.ActorLogging
import gnu.io._
import src.ship._

class EmergencyButtonActor() extends ActWithStash with ActorLogging {

  class SerialReader(s: SerialPort) extends SerialPortEventListener {
    def serialEvent(arg0: SerialPortEvent) {
      var sit: String = if (s.isCD) "Normal" else "Emergency"
      log.info(sit)
      SHIPTool.ref ! Update(List[ABoxFormula](ConceptAssertion(Const("situation"), ConceptConstant(sit))), List[Const]())
    }
  }

  override def receive = {
    case _ =>
  }

  override def preStart() {
    var port = if (System.getProperty("os.name").startsWith("Windows")) "COM5"
    else if (System.getProperty("os.name").startsWith("Mac OS X")) "/dev/tty.usbserial"
    else "/dev/ttyUSB0"
    log.info("trying to start emergency button actor at " + port)

    try {
      val portIdentifier: CommPortIdentifier = CommPortIdentifier.getPortIdentifier(port)
      if (portIdentifier.isCurrentlyOwned) {
        log.error("Port is currently in use:" + port)
      }
      else {
        val commPort: CommPort = portIdentifier.open(this.getClass.getName, 2000)
        commPort match {
          case serialPort: SerialPort =>
            if (!serialPort.isCD) log.warning("Release Emergency Button!")
            serialPort.addEventListener(new SerialReader(serialPort))
            serialPort.notifyOnCarrierDetect(true)
          case _ =>
            log.error("Only serial ports are handled by this example:" + port)
        }
      }
    } catch {
      case e: NoSuchPortException => log.warning("No such port for Emergency Button: " + port)
      case e: NoClassDefFoundError => log.warning("RXTX library not found")
      case e: Throwable => log.error(e, "unexpected exception")
    }
  }
}
