package src.BAALL

import akka.actor.ActorDSL._
import akka.actor.{Props, ActorLogging, ActorRef}
import java.io.IOException
import java.net.UnknownHostException
import src.ship._

/**
 * Created with IntelliJ IDEA.
 * User: dodi
 * Date: 11.09.12
 * Time: 14:57
 * bidirectional connection to a LKNX Object
 */
class Visualizer(name: String, port: Int, isYamamoto: Boolean, shipTool: ActorRef)
  extends ActWithStash with ActorLogging {

  val lknxconnection = new Linknx(name, port)
  val registeredObjects = new scala.collection.parallel.mutable.ParHashMap[String, LKNXObject]
  val OntUpdater = new OntUpdater(lknxconnection, shipTool, registeredObjects)
  //val ta = new TimerActor(OntUpdater, 1000, isYamamoto)
  val timerActor = context.actorOf(Props(new TimerActor(OntUpdater, 1000, isYamamoto)), "timer" + port)
  val LKNXUpdater = context.actorOf(Props(new LKNXUpdater(name, port, OntUpdater, registeredObjects, lknxconnection)), "KNX" + port)

  override def preStart() {
    log.info("Visualizer started *** " + name + ":" + port)
    try {
      lknxconnection.connect()
      for (obj <- lknxconnection.readAllObjects()) registeredObjects.put(lknxconnection.getName(obj), obj)
      log.info("Registered objects:\n" + registeredObjects)
      if (isYamamoto) lknxconnection.sendRouteGraph()
    }
    catch {
      case x: UnknownHostException => log.error(x, "Could not connect to host " + name + ":" + port)
      case x: IOException => log.error(x, "I/O Exception on socket with " + name + ":" + port)
      /* case x: Throwable => log.error(x, "Exception on " + name + ":" + port + " " + x)
        throw x */
    }
  }

  override def receive = {
    case v@UCHPolling(_) => if (isYamamoto) LKNXUpdater ! v
    case v@EmptyOntology =>
      registeredObjects.clear()
      for (obj <- lknxconnection.readAllObjects()) {
        log.warning("registering " + lknxconnection.getName(obj) + " = " + obj)
        registeredObjects.put(lknxconnection.getName(obj), obj)
      }
      LKNXUpdater ! v
    case v@CurrentOntology(o: Ontology) =>
      LKNXUpdater ! v
    case RollandPosition2D(n, p) =>
      log.info("Visualizer for " + name + " obtained new Position " + p + " for " + n)
      if (isYamamoto) {
        lknxconnection.write2DPosition(n, p)
        if (OntUpdater.ontology != null) {
          OntUpdater.ontology.query("(" + n + ",?person):wheelchair_carries, ?person:Person") match {
            case s :: _ =>
              val pname = s(Var("person")).asInstanceOf[Const].name
              lknxconnection.write2DPosition(pname, p)
            case _ =>
          }
        }
      }
    //case RollandNewRouteGraph(n,g) => // log.info("Visualizer for "+name+" obtained new route graph from "+n)
    case src.ship.Stop() =>
      log.warning("Visualizer recieved Stop")
      timerActor ! src.ship.Stop()
    case _ =>
  }
}
