package src.ship

import akka.actor.ActorDSL._
import java.awt.Font
import swing.{Table, ScrollPane}

/**
 * Created with IntelliJ IDEA.
 * User: autexier
 * Date: 25.06.2012
 * Time: 11:22
 */

class ProcessesPane(textfont: Font) extends ScrollPane with JavaLogging {
  var data = Array.tabulate[Any](0, 2)((_, _) => "")
  val headers = scala.collection.mutable.Seq("Process", "Status")
  var table = new Table(data, headers) {
    font = textfont
    peer.getTableHeader.setFont(textfont)
    autoResizeMode = Table.AutoResizeMode.AllColumns
    rowHeight = (15 * textfont.getSize) / 12

  }
  //autoResizeMode = Table.AutoResizeMode.AllColumns
  viewportView = table
  var programrow = Map.empty[DLProgram, Int]

  def updateTable(p: DLProgram, longStatus: String) {
    val oldEntry = programrow.get(p)
    val status = if (longStatus.length() > 60) longStatus.substring(0, 60) + "..." else longStatus
    log.info("update: " + p + " " + status)
    oldEntry match {
      case None =>
        data = Array.tabulate(table.rowCount + 1, 2)((x, y) => {
          //println("x = "+x+" y = "+y)
          if (x == table.rowCount) {
            if (y == 0) p else status
          }
          else {
            data(x)(y)
          }
        })
      case Some(z) =>
        data = Array.tabulate(table.rowCount, 2)((x, y) => {
          if (x == table.rowCount - 1) {
            if (y == 0) p else status
          }
          else {
            if (x < z) data(x)(y)
            else {
              if (y == 0) programrow = programrow updated(data(x + 1)(0).asInstanceOf[DLProgram], x)
              data(x + 1)(y)
            }
          }
        })
    }
    table = new Table(data, headers) {
      font = textfont
      peer.getTableHeader.setFont(textfont)
      autoResizeMode = Table.AutoResizeMode.AllColumns
      rowHeight = (15 * textfont.getSize) / 12
    }
    programrow = programrow updated(p, table.rowCount - 1)
    viewportView = table
  }

  def resetTable(ml: List[DLProgram]) {
    log.info("reset table")
    if (ml.isEmpty) {
      data = Array.tabulate[Any](0, 2)((_, _) => "")
      table = new Table(data, headers) {
        font = textfont
        peer.getTableHeader.setFont(textfont)
        autoResizeMode = Table.AutoResizeMode.AllColumns
        rowHeight = (15 * textfont.getSize) / 12

      }
      viewportView = table
      programrow = Map.empty[DLProgram, Int]
    } else {
      var newprogramrow = Map.empty[DLProgram, Int]
      var newdata = Array.tabulate[Any](table.rowCount - ml.length, 2)((_, _) => "")
      var newrow = 0
      (0 until table.rowCount - 1) map (x => {
        ml find (p => programrow.get(p) == Some(x)) match {
          case None => {}
          case Some(p) => {
            newprogramrow = newprogramrow updated(p, newrow)
            table.update(newrow, 0, p)
            table.update(newrow, 1, data(x)(1))
            newrow = newrow + 1
          }
        }
      })

      data = newdata
      programrow = newprogramrow

      table = new Table(data, headers) {
        autoResizeMode = Table.AutoResizeMode.AllColumns
        font = textfont
        peer.getTableHeader.setFont(textfont)
        rowHeight = (15 * textfont.getSize) / 12

      }
      viewportView = table
    }
  }
}

class SHIPGUIProcesses(sp: ProcessesPane) extends ActWithStash {
  override def receive = {
    case ProgramError(p, msg) => sp.updateTable(p, msg)
    case ProgramSuccess(p) => sp.updateTable(p, "Done")
    case ProgramFailure(p) => sp.updateTable(p, "Failed")
    case ProgramStutter(p, rest) => sp.updateTable(p, "Stutter " + rest)
    case ProgramUpdateAbox(p, Done(_, _)) => sp.updateTable(p, "Done")
    case ProgramUpdateAbox(p, Stutter(_, _, _)) => sp.updateTable(p, "Stutter")
    case ProgramUpdateAbox(p, Step(_, _, _)) => sp.updateTable(p, "Step")
    case ProcessesReset(_) => sp.resetTable(Nil)
    case _ =>
  }
}
