package src.ship.external

import scala.List
import scala.collection.mutable
import src.ship._

/**
 * Created by IntelliJ IDEA.
 * User: dodi
 * Date: 7/27/12
 * Time: 11:46 AM
 */

object RouteComputation extends JavaLogging {

  def computeAllRoutes(start: String, points: List[String], neighbours: List[(String, String)]): (mutable.HashMap[String, Int], mutable.HashMap[String, String]) = {
    // println("computeAllRoutes " + start + " " + points + " " + neighbours)
    val distances: mutable.HashMap[String, Int] = new mutable.HashMap()
    val predecessors: mutable.HashMap[String, String] = new mutable.HashMap()
    for (p <- points) distances.put(p, 10000)
    distances.put(start, 0)
    var notvisited = mutable.MutableList[String]() ++ points
    var current = start
    var nextunvisitedexists = true
    while (!notvisited.isEmpty && nextunvisitedexists) {

      //println("current = "+current)
      //println("notvisited = "+notvisited+" "+notvisited.isEmpty)
      //println("nextunvisistedexists "+nextunvisitedexists)
      val nextpositions: List[String] = (neighbours :\ List[String]())((p, l) => if (p._1 == current) p._2 :: l else l)
      //println("next positions "+nextpositions)
      for (p <- nextpositions) {
        val dist = distances.get(current).get + 1
        if (dist < distances.get(p).get) {
          distances.put(p, dist)
          predecessors.put(p, current)
        }
      }
      notvisited = notvisited.filterNot(current.equals(_))
      // find next node to visit
      var nextdist = 10000
      nextunvisitedexists = false
      for (p <- notvisited)
        if (distances.get(p).get < nextdist) {
          nextdist = distances.get(p).get
          current = p
          nextunvisitedexists = true
        }
    }
    (distances, predecessors)
  }


  def selectClosestWheelChair(ont: Ontology, pos: String, subject: String): Option[String] = {
    ont.query("(wc,:" + subject + "):wheelchair_carries") match {
      //ont.query(NamedRole("wheelchair_carries")(Var("wc"),Const(subject))) match {
      case subst :: _ => Some(subst(Var("wc")).asInstanceOf[Const].name)
      case Nil => {
        //ont.query(List(ConceptConstant("WCAvailable")(Var("wc")),NamedRole("at")(Var("wc"),Var("pos")))) match {
        ont.query("wc:WCAvailable,(wc,pos):at") match {
          case Nil => None
          case wcs => {
            log.info("substs: " + wcs)
            val points = ont.allIndividuals(ConceptConstant("Position"))
            val neighbours = Ontology2Scala.RoleEntriesAsList(ont, NamedRole("neighbour"))
            val (dist, _) = computeAllRoutes(pos, points, neighbours)
            //println(dist)
            var map: List[(String, Int)] = List()
            for (wc <- wcs) map = (wc(Var("wc")).asInstanceOf[Const].name,
              dist.get(wc(Var("pos")).asInstanceOf[Const].name).get) :: map
            map = map.sortBy(x => x._2)
            log.info("Closest map: " + map)
            if (map.length > 1 && map.head._2 == map.tail.head._2 && map.tail.head._1.equalsIgnoreCase("Rolland2")) Some(map.tail.head._1)
            else Some(map.head._1)
          }
        }
      }
    }
  }

  def print(ont: Ontology, goal: String): Option[UnconditionalEffect] = {
    log.info("Individual "+goal)
    Some(UnconditionalEffect(Nil,Nil))
  }

  def assignClosestWC(ont: Ontology, goal: String): Option[UnconditionalEffect] = {
    log.info("assignClosestWC")
    ont.query(
      "mode:WCPickup,(:" + goal + ",subject):goal_subject,(subject,target):at,(target,pickuppos):pickup_position"
      /*List(
        ConceptConstant("WCPickup")(Var("mode")),
        NamedRole("goal_subject")(Const(goal),Var("subject")),
        NamedRole("at")(Var("subject"),Var("target")),
        NamedRole("pickup_position")(Var("target"),Var("pickuppos"))
      )*/) match {
      case Nil => None
      case subst :: _ => {
        //println("subst "+subst)
        val target = subst(Var("pickuppos")).asInstanceOf[Const].name
        val subject = subst(Var("subject")).asInstanceOf[Const].name

        selectClosestWheelChair(ont, target, subject) match {
          case None => None
          case Some(closestWC) =>
            ont.query(
              "(:" + closestWC + ",source):at"
              //NamedRole("at")(Const(closestWC),Var("source"))
            ) match {
              case Nil => None
              case subst2 :: _ => {

                val wcpos: String = subst2(Var("source")).asInstanceOf[Const].name
                val planrequest = ont.newIndividualGenerator
                val mode: String = subst(Var("mode")).asInstanceOf[Const].name

                val newabox: List[ABoxFormula] = List(
                  NamedRole("wheelchair_mode").roleAssert(Const(closestWC), Const(mode)),
                  NamedRole("goal_wheelchair").roleAssert(Const(goal), Const(closestWC)),
                  //ConceptConstant("PlanRequest")(Const(planrequest)),
                  NamedRole("planrequest_source").roleAssert(Const(planrequest), Const(wcpos)),
                  NamedRole("planrequest_target").roleAssert(Const(planrequest), Const(target)),
                  NamedRole("planrequest_wheelchair").roleAssert(Const(planrequest), Const(closestWC)))
                log.info("new abox: " + newabox)
                Some(UnconditionalEffect(newabox, List()))
              }
            }
        }
      }
    }
  }

  def sendWCToClosestNonBlockingPosition(ont: Ontology, wc:String, pos:String):Option[UnconditionalEffect] = {
    val points = ont.allIndividuals(ConceptConstant("Position"))
    val neighbours = Ontology2Scala.RoleEntriesAsList(ont, NamedRole("neighbour"))

    val (dist, _) = computeAllRoutes(pos, points, neighbours)
    val map = dist.toList.sortBy(x => x._2)
    //log.info("closest positions "+map)
    val conflictmap = PathCoordination.getCurrentConflictMap
    //log.info("conflictmap "+conflictmap)
    val blockingpos = ont.query(ConceptConstant("PositionIsBlocking").mkAssertion(Var("x"))).map { subst =>
      subst.apply(Var("x")).asInstanceOf[Const].name}
    val edges = ont.query(
      List(ConceptConstant("PositionIsBlocking").mkAssertion(Var("x")),
           ConceptConstant("PositionIsBlocking").mkAssertion(Var("y")),
           NamedRole("route_next").roleAssert(Var("z"),Var("x")),
           NamedRole("route_rest").roleAssert(Var("z"),Var("u")),
           NamedRole("route_next").roleAssert(Var("u"),Var("y")))).map { subst =>
             Pair(subst(Var("x")).asInstanceOf[Const].name,
                  subst(Var("y")).asInstanceOf[Const].name)}

    val person = ont.query(List(NamedRole("wheelchair_carries").roleAssert(Const(wc),Var("person")),
                               ConceptConstant("Person").mkAssertion(Var("person")))) match {
      case subst :: _ => subst.apply(Var("person")).asInstanceOf[Const]
      case Nil => Const("nobody")
    }


    //log.info("blockingpos "+blockingpos)
    val tmp = map find {entry =>
      !blockingpos.contains(entry._1) &
        (blockingpos forall { x => !conflictmap.conflicts(entry._1,x)}) &
        (edges forall { edge => !conflictmap.conflicts(entry._1,edge._1,edge._2)})}

    //log.info("selected "+tmp)
    tmp match {
      case Some((newpos,d)) =>
        val planrequest = ont.newIndividualGenerator
        val goal = ont.newIndividualGenerator
        val newabox: List[ABoxFormula] = List(
          NamedRole("goal_wheelchair").roleAssert(Const(goal), Const(wc)),
          ConceptConstant("None").mkAssertion(person),
          NamedRole("goal_target").roleAssert(Const(goal), Const(newpos)),
          NamedRole("goal_subject").roleAssert(Const(goal), person),
          NamedRole("planrequest_source").roleAssert(Const(planrequest), Const(pos)),
          NamedRole("planrequest_target").roleAssert(Const(planrequest), Const(newpos)),
          NamedRole("planrequest_wheelchair").roleAssert(Const(planrequest), Const(wc)))
        log.info("new abox: " + newabox)
        Some(UnconditionalEffect(newabox, List()))
      case None => None
    }
  }


  /**
   *
   * @param position    dfg
   * @param points         dfsg
   * @param neighbours     dfgs
   */
  def findClosestStopPosition(position: String, points: List[String], neighbours: List[(String, String)], forbidden: List[String], targets: List[String]) = {
    val (dist, _) = computeAllRoutes(position, points, neighbours)
    var map: List[(String, Int)] = List()
    for (target <- targets) if (!forbidden.contains(target)) map = (target, dist.get(target).get) :: map
    map = map.sortBy(x => x._2)
    //println("sorted map "+map)
    if (map.size > 0) map.iterator.next()._1
    else null
  }

  /**
   *
   */
  def computeRoute(start: String, target: String, points: List[String], neighbours: List[(String, String)]): List[String] = {
    //println("computeRoute "+start+" "+target+" "+points+" "+neighbours)
    val preds = computeAllRoutes(start, points, neighbours)._2
    var current = target
    var l = List[String]()
    while (current != start) {
      l = current :: l
      current = preds.get(current).get
    }
    l
  }

  def computeRouteIntern(ont: Ontology, rstart: String, rend: String, planrequest: String,
                         oldroute: String, wheelchair: String): Option[UnconditionalEffect] = {
    var aboxupdate = List.empty[ABoxFormula]
    var deleteind = List.empty[Const]

    val points = ont.allIndividuals(ConceptConstant("Position"))
    val neighbours = Ontology2Scala.RoleEntriesAsList(ont, NamedRole("neighbour"))

    val route = RouteComputation.computeRoute(rstart, rend, points, neighbours)
    val (routeont, routehead) =
      Scala2Ontology.list2Abox(route, "", "route_rest", "route_next", "EmptyRoute", "Route", ont)
    log.info("compute route for " + wheelchair + " from " + rstart + " to " + rend + "\nresults in " + route.mkString(",") + " ont: " + routeont + " with head " + routehead + ont.allInds)
    aboxupdate = aboxupdate ++ (RoleAssertion(Const(wheelchair), Const(oldroute), NotRole(NamedRole("wheelchair_route"))) ::
      RoleAssertion(Const(wheelchair), Const(routehead), NamedRole("wheelchair_route")) :: routeont)
    deleteind = Const(planrequest) :: deleteind
    // println("WCPlanner update\nDelete "+deleteind+"\nABOXUpdate "+aboxupdate)
    if (aboxupdate.isEmpty && deleteind.isEmpty) None
    else Some(UnconditionalEffect(aboxupdate, deleteind))
  }

  def deleteRequestAndRoute(ont: Ontology, request: String, wc: String,
                            rstart: String, emptyroute: String, wcidle: String): Option[UnconditionalEffect] = {
    var deleteind: List[String] = List(request)
    // 1. Deleting the route and resetting wheelchair route
    var current = rstart
    // val nonemptyRoute = ConceptConstant("NonEmptyRoute")
    // val routerestRole = NamedRole("route_rest")

    while ( //!ont.query(nonemptyRoute(Const(current))).isEmpty
      !ont.query(":" + current + ":NonEmptyRoute").isEmpty
    ) {
      deleteind = current :: deleteind
      //current = ont.query(routerestRole(Const(current),Var("next"))).head(Var("next")).asInstanceOf[Const].name
      current = ont.query("(:" + current + ",next):route_rest").head(Var("next")).asInstanceOf[Const].name
    }
    log.info("ROUTE: Deleting " + deleteind)
    var updates: List[ABoxFormula] = Nil
    if (deleteind.length > 1) {
      updates = NamedRole("wheelchair_route").roleAssert(Const(wc), Const(emptyroute)) :: updates
    }

    // 2. If the wheelchair does not carry anybory, make him idle

    if (ont.query(
      "(:" + wc + ",person):wheelchair_carries"
      //NamedRole("wheelchair_carries")(Const(wc),Var("person"))
    ).isEmpty) {
      // there isn't anybody on the wheelchair
      updates = NamedRole("wheelchair_mode").roleAssert(Const(wc), Const(wcidle)) :: updates
    }

    // 3. If the are positions reseved for the wheelchair, but it isn't there.
    Some(UnconditionalEffect(updates, deleteind map (n => Const(n))))
  }
}


object WCPlanner extends SHIPToolClient with JavaLogging {
  var lastrequests: mutable.HashMap[String, (String, String)] = new mutable.HashMap[String, (String, String)]()

  ontcase(
    "r:PlanRequest, (r,wc):planrequest_wheelchair, (r,src):planrequest_source, " +
      "(wc,oldroute):wheelchair_route, (r,trg):planrequest_target",
    computeRouteAndUpate)

  /**
   * computes a route for those route requests that do not yet have a route.
   * param: a substitution, where w is bound to the wc that poses the request
   * planrequest_source(p,s) is bound to the current positition of the wheelchair
   * planrequest_target(p,t) pend is the target position
   * r is bound to the current route
   */

  def computeRouteAndUpate(ont: Ontology, substs: List[Substitution]): Option[Pair[List[ABoxFormula], List[Const]]] = {
    var aboxupdate = List.empty[ABoxFormula]
    var deleteind = List.empty[Const]

    //val newindgen = ont.newIndividualGenerator

    for (subst: Substitution <- substs) {
      val rstart = subst.apply(Var("src")).name
      val rend = subst.apply(Var("trg")).name
      val planrequest = subst.apply(Var("r")).name
      //println("start "+rstart+" end " +rend)
      val oldroute = subst.apply(Var("oldroute")).name
      val wheelchair = subst.apply(Var("wc")).name
      val points = ont.allIndividuals(ConceptConstant("Position"))
      val neighbours = Ontology2Scala.RoleEntriesAsList(ont, NamedRole("neighbour"))
      // println("positions: "+points)
      // println("neighbours: "+neighbours)
      val lastinfo = lastrequests.get(wheelchair)
      if (!lastinfo.isDefined || lastinfo.get._1 != rstart || lastinfo.get._2 != rend) {
        lastrequests.put(wheelchair, (rstart, rend))
        val route = RouteComputation.computeRoute(rstart, rend, points, neighbours)
        val (routeont, routehead) =
          Scala2Ontology.list2Abox(route, "", "route_rest", "route_next", "EmptyRoute", "Route", ont)
        log.info("compute route for " + wheelchair + " from " + rstart + " to " + rend + "results in\n " +
          route.mkString(",") + "\nont: " + routeont + "\nwith head " + routehead + " " + ont.allInds)
        // println("neighbours: "+neighbours+" reounte computation results in "+route)
        aboxupdate = aboxupdate ++ (RoleAssertion(Const(wheelchair), Const(oldroute), NotRole(NamedRole("wheelchair_route"))) ::
          RoleAssertion(Const(wheelchair), Const(routehead), NamedRole("wheelchair_route")) :: routeont)
        deleteind = Const(planrequest) :: deleteind
      }
      else log.warning("Skip route computation from " + rstart + " to " + rend + " as double request")
    }
    // println("WCPlanner update\nDelete "+deleteind+"\nABOXUpdate "+aboxupdate)
    if (aboxupdate.isEmpty && deleteind.isEmpty) None
    else Some(Pair(aboxupdate, deleteind))
  }
}
