package src.simple

import java.awt.Font

import akka.actor.ActorDSL._
import akka.actor.{ActorDSL, ActorRef, ActorSystem}
import src.ship._

import scala.swing.Swing._
import scala.swing._
import scala.swing.event._

/**
 * Created with IntelliJ IDEA.
 * User: bawadekji
 * Date: 4/4/13
 * Time: 5:35 PM
 */


case class CreatePatientMonitor(patient: String)

case class SelectPatientDialog(patient: String)

class SimpleGUI(font: swing.Font, titlestr: String) extends ActWithStash with SHIPToolParser {
  val monitorsPane = new MonitorsPane(font)
  val system = ActorSystem("migration-system")
  val shipguimonitors = ActorDSL.actor(system)(new SHIPGUIMonitors(monitorsPane))
  var Helper = ActorDSL.actor(new SimpleGUIHelper(shipguimonitors))


  val frame = new Frame()
  frame.title = "Projekt SIMPLE - System Demonstrator "
  var ont: Option[Ontology] = None
  var patients = List[String]()
  //var boxes = new SimpleBoxPanel(self, monitorsPane, patients)
  var boxes = new SimpleBoxPanel(self, patients)
  frame.contents = boxes
  // system setup
  SHIPTool.ref ! "load examples/simple/acs.ont"
  SHIPTool.ref ! "load examples/simple/acs.prg"
  SHIPTool.ref ! "load examples/simple/acs.mon"
  // end system setup
  frame.open()

  // var PatientMap: Map[String, Pair[Option[Frame], Option[ActorRef]]] =  Map()
  var PatientMap: Map[String, (Option[Frame], Option[ActorRef])] = Map()

  var test = new MonitorProgress(_, _, _)

  override def receive = {

    case d@MonitorFailure(phi, _, _, _, _) =>
      //  Helper ! d
      println("$$$$$$$$$$$$$$$$$$$$$$$$$$  " + phi)
      phi match {
        case DefPred(Pred(_, patientnm :: _)) =>
          PatientMap get patientnm.toString match {
            case Some((_, monitorActor)) =>
              monitorActor match {
                case Some(monActor) =>
                  monActor ! d
                case None =>
              }
            case None =>
          }
        case _ =>
      }

    case v@MonitorProgress(phi, current, desc) =>
      Helper ! v

      //   test =   MonitorProgress(phi, current, desc)
      //    println("$$$$$$$$$$$$$$$$$$$$$$$$$$  "+ phi+"   "+ desc + "  "+ current )
      phi match {
        case DefPred(Pred(_, patientnm :: _)) =>
          PatientMap get patientnm.toString match {

            case Some((_, monitorActor)) =>
              monitorActor match {
                case Some(monActor) =>
                  monActor ! v
                case None =>
              }

            case None =>
          }
        case _ =>
      }
    case CurrentOntology(o) => println("CurrentOntology" + o)
      ont = Some(o)

    case CreatePatientMonitor(patient) =>

      var monitorsPane1 = new MonitorsPane(font)
      var shipguimonitors1 = ActorDSL.actor(system)(new SHIPGUIMonitors(monitorsPane1))


      val patientFrame = new Frame()
      patientFrame.contents = new PatientBoxPanel(self, monitorsPane1, patient)

      patientFrame.title = "Monitoring the medical care of patient    " + patient
      patientFrame.centerOnScreen()
      //  patientFrame.open()
      //  patientFrame.visible = true
      PatientMap += ((patient, (Option(patientFrame), Option(ActorDSL.actor(new SimpleGUIHelper(shipguimonitors1))))))

    case SelectPatientDialog(patient) =>
      PatientMap get patient match {
        case Some((pframe, _)) =>
          pframe match {
            case Some(fr) =>
              //   fr.visible    =  true
              fr.open()
            case None =>
          }
        case None =>
      }
    case s: String =>
      ont match {
        case None =>
        case Some(o) =>
          o.query(":" + s + ":Patient") match {
            case Nil =>
              val Cmd: String = "update  " + s + ":Patient"
              parseAll(ercmd, Cmd) match {
                case Success(result, _) =>
                  SHIPTool.ref ! "update  " + s + ":Patient"
                  SHIPTool.ref ! "run Treatment" + "(" + s + ")"

                case _ =>
              }

              patients = s :: patients
              //  boxes = new SimpleBoxPanel(self, monitorsPane, patients)
              boxes = new SimpleBoxPanel(self, patients)
              frame.contents = boxes
              frame.close()

              frame.open()
            case _ => println("Patient is registered in the system")
          }
      }
    case _ =>
  }
}


//case class SimpleBoxPanel(cb: ActorRef, pane: MonitorsPane, patients: List[String]) extends BoxPanel(Orientation.Vertical) {
case class SimpleBoxPanel(cb: ActorRef, patients: List[String]) extends BoxPanel(Orientation.Vertical) {
  var fontsize = 12
  val labelfont = new swing.Font(null, Font.BOLD, fontsize)
  val textfont = new swing.Font(null, Font.PLAIN, fontsize)

  val b1 = new Button {
    text = "Add Patient"
  }
  val b2 = new Button {

    text = "Monitor"
  }
  val b3 = new Button {
    text = "Start Treatment"
  }
  val b4 = new Button {
    text = "Start Monitor"

  }
  val b5 = new Button {
    text = "Increment Time"
  }
  val b6 = new Button {
    text = "Update"
  }
  var textfield = new TextField {
    text = "                     "
  }
  var items = patients
  //var checks = checkboxes map ((s:String) => new CheckBox(s))

  val DiagnosisItems = List[String]("Unknown", "STEMI", "NSTEMI", "STABLE")
  val SymptomItems = List[String]("Unknown", "Symptoms for Revascularisation", "Symptoms for OMT", "NonProximal LAD", "No other Symptoms", "Other Symptoms")
  val ProcedureItems = List[String]("Unknown", "OMT", "PCI", "CABG", "Fibrinolysis", "Coronary Angiography")
  val TherapyItems = List[String]("Unknown", "Aspirin", "Clopidogrel", "GP IIb", "Heparin", "Bivalirudin")


  val ContraindicationItems = List[String]("Unknown", " Contraindication For Aspirin", "Known bleeding disorder", "Aortic dissection", "Central nervous system " +
    "trauma or neoplasms", "Non-compressible punctures", "Oral anticoagulant therapy", "Pregnancy or 1 week post-partum",
    "Advanced liver disease", "Active peptic ulcer", "Refractory resuscitation")

  val PatientCombo: ComboBox[String] = new ComboBox[String](items)

  val DiagnosisCombo: ComboBox[String] = new ComboBox[String](DiagnosisItems) {
    preferredSize = new Dimension(260, 25)
    maximumSize = preferredSize
    minimumSize = preferredSize
  }
  val SymptomCombo: ComboBox[String] = new ComboBox[String](SymptomItems) {
    preferredSize = new Dimension(260, 25)
    maximumSize = preferredSize
    minimumSize = preferredSize
  }
  val ProcedureCombo: ComboBox[String] = new ComboBox[String](ProcedureItems) {
    preferredSize = new Dimension(260, 25)
    maximumSize = preferredSize
    minimumSize = preferredSize
  }

  val TherapyCombo: ComboBox[String] = new ComboBox[String](TherapyItems) {
    preferredSize = new Dimension(260, 25)
    maximumSize = preferredSize
    minimumSize = preferredSize
  }
  val ContraindicationCombo: ComboBox[String] = new ComboBox[String](ContraindicationItems) {
    preferredSize = new Dimension(260, 25)
    maximumSize = preferredSize
    minimumSize = preferredSize
  }


  def check(s: String): Boolean = if (s.length > 0) true else false

  contents ++= List(
    new BoxPanel(Orientation.Horizontal) {
      contents += textfield
    },
    new BoxPanel(Orientation.Vertical) {
      contents += PatientCombo
      contents += VStrut(10)
    },
    new BoxPanel(Orientation.Horizontal) {
      contents += new Label("Select Diagnosis                                   ")
      contents += DiagnosisCombo
      contents += VStrut(30)
    },
    new BoxPanel(Orientation.Horizontal) {
      contents += new Label("Select Symptom                                    ")
      contents += SymptomCombo
      contents += VStrut(30)
    },
    new BoxPanel(Orientation.Horizontal) {
      contents += new Label("Select Procedure                                  ")
      contents += ProcedureCombo
      contents += VStrut(30)
    },
    new BoxPanel(Orientation.Horizontal) {

      contents += new Label("Select Therapy                                      ")
      contents += TherapyCombo
      contents += VStrut(30)
    },
    new BoxPanel(Orientation.Horizontal) {
      contents += new Label("Select Contraindication                       ")
      contents += ContraindicationCombo
      contents += VStrut(30)
    },
    new BoxPanel(Orientation.Vertical) {
      //   contents ++= checks
      contents += VStrut(10)
      // contents += pane
      contents += VStrut(10)
    },
    new BoxPanel(Orientation.Horizontal) {
      contents += VStrut(10)
      contents += b1 //new BorderPanel { add(b1, BorderPanel.Position.West)  }
      contents += VStrut(10)
      contents += b2
      contents += VStrut(10)
      contents += b5
      contents += VStrut(10)
      contents += b6
    }
  )

  var PatientName = ""
  var Procedure = ""
  var Symptom = ""
  var RunCommand = ""

  reactions += {
    case event.ButtonClicked(`b1`) =>
      val text = textfield.text.replaceAll("\\s", "")
      if (text.length > 0) {
        cb ! text
        PatientName = text
      }
      cb ! CreatePatientMonitor(PatientName)
    //  case event.ButtonClicked(`b3`) if PatientName.length > 0  =>

    //     SHIPTool.ref ! "run Treatment" + "(" + PatientName + ")"

    //  val pGui = new PGUI(PatientName)
    //  val pGui  = ActorDSL.actor(new PGUI(PatientName))
    //    val a = ActorDSL.actor(pGui.newInstance(PatientName).asInstanceOf[ActWithStash])

    //  val stableGui = new StableGUI(PatientName)
    //  stableGui.visible = true


    case event.ButtonClicked(`b2`) if PatientName.length > 0 =>
      cb ! SelectPatientDialog(PatientName)

    case event.ButtonClicked(`b5`) =>
      SHIPTool.ref ! "run IncrementTherapyDoseTimer" + "(" + PatientName + ")"

    case event.ButtonClicked(`b6`) if PatientName.length > 0 && RunCommand.length > 0 =>
      SHIPTool.ref ! RunCommand
      println("Update Command is passed " + RunCommand)
      RunCommand = ""


    case SelectionChanged(`DiagnosisCombo`) if PatientName.length > 0 => DiagnosisCombo.selection.item match {
      case _ if DiagnosisCombo.selection.item == DiagnosisItems(0) => println(DiagnosisItems(0))
      case _ if DiagnosisCombo.selection.item == DiagnosisItems(1) => println(DiagnosisItems(1))
      case _ if DiagnosisCombo.selection.item == DiagnosisItems(2) => println(DiagnosisItems(2))
      case _ if DiagnosisCombo.selection.item == DiagnosisItems(3) =>
        RunCommand = "run Diagnose" + DiagnosisItems(3) + "(" + PatientName + ")"
      //   val stableGui = new StableGUI(PatientName)
      //   stableGui.visible = true
    }
    case SelectionChanged(`SymptomCombo`) if PatientName.length >= 0 => SymptomCombo.selection.item match {
      case _ if SymptomCombo.selection.item == SymptomItems(0) => println("Select a Symptom")
      case _ if SymptomCombo.selection.item == SymptomItems(1) => RunCommand = "run " + SymptomItems(1).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if SymptomCombo.selection.item == SymptomItems(2) => RunCommand = "run " + SymptomItems(2).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if SymptomCombo.selection.item == SymptomItems(3) => RunCommand = "run " + SymptomItems(3).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if SymptomCombo.selection.item == SymptomItems(4) => RunCommand = "run " + SymptomItems(4).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if SymptomCombo.selection.item == SymptomItems(5) => RunCommand = "run " + SymptomItems(5).replaceAll("\\s+", "") + "(" + PatientName + ")"
    }

    case SelectionChanged(`ProcedureCombo`) if PatientName.length > 0 => ProcedureCombo.selection.item match {
      case _ if ProcedureCombo.selection.item == ProcedureItems(0) => println("Select a Procedure")
      case _ if ProcedureCombo.selection.item == ProcedureItems(1) => RunCommand = "run  Prescribe" + ProcedureItems(1).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if ProcedureCombo.selection.item == ProcedureItems(2) => RunCommand = "run  Prescribe" + ProcedureItems(2).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if ProcedureCombo.selection.item == ProcedureItems(3) => RunCommand = "run  Prescribe" + ProcedureItems(3).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if ProcedureCombo.selection.item == ProcedureItems(4) => RunCommand = "run  Prescribe" + ProcedureItems(4).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if ProcedureCombo.selection.item == ProcedureItems(5) => RunCommand = "run  Prescribe" + ProcedureItems(5).replaceAll("\\s+", "") + "(" + PatientName + ")"
    }
    case SelectionChanged(`TherapyCombo`) if PatientName.length > 0 => TherapyCombo.selection.item match {
      case _ if TherapyCombo.selection.item == TherapyItems(0) => println("Select a Therapy")
      case _ if TherapyCombo.selection.item == TherapyItems(1) => RunCommand = "run  Prescribe" + TherapyItems(1).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if TherapyCombo.selection.item == TherapyItems(2) => RunCommand = "run  Prescribe" + TherapyItems(2).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if TherapyCombo.selection.item == TherapyItems(3) => RunCommand = "run  Prescribe" + TherapyItems(3).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if TherapyCombo.selection.item == TherapyItems(4) => RunCommand = "run  Prescribe" + TherapyItems(4).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if TherapyCombo.selection.item == TherapyItems(5) => RunCommand = "run  Prescribe" + TherapyItems(5).replaceAll("\\s+", "") + "(" + PatientName + ")"

    }
    case SelectionChanged(`ContraindicationCombo`) if PatientName.length > 0 => ContraindicationCombo.selection.item match {
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(0) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(1) => RunCommand = "run " + ContraindicationItems(1).replaceAll("\\s+", "") + "(" + PatientName + ")"
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(2) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(3) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(4) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(5) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(6) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(7) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(8) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(9) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(10) => println("Select another Contraindication ")
      case _ if ContraindicationCombo.selection.item == ContraindicationItems(11) => println("Select another Contraindication ")

    }

    case SelectionChanged(`PatientCombo`) =>
      PatientName = PatientCombo.selection.item
  }

  border = Swing.EmptyBorder(30, 30, 10, 30)
  listenTo(b1, b2, b5, b6, textfield, PatientCombo.selection, DiagnosisCombo.selection, SymptomCombo.selection,
    ProcedureCombo.selection, TherapyCombo.selection, ContraindicationCombo.selection)
}


class SimpleGUIHelper(target: ActorRef) extends ActWithStash {
  override def receive = {
    case c@MonitorProgress(_, _, _) => println("SimpleGUIHelper==========" + c)
      //  if (List("m1").contains(monitorname))
      target ! c
    case d@MonitorFailure(phi, _, _, _, _) =>
      target ! d
    case _ =>
  }
}


case class PatientBoxPanel(cb: ActorRef, pane: MonitorsPane, p: String) extends BoxPanel(Orientation.Horizontal) {
  contents ++= List(
    new BoxPanel(Orientation.Vertical) {
      contents += pane
    }
  )
}


/*

class StableGUI(s: String)    extends Frame {
  var fontsize = 12
  val textfont = new swing.Font(null, Font.PLAIN, fontsize)
  val monitorsPane = new MonitorsPane(textfont)

  title = "Stable CAD Sym. of Patient "  + s
 // contents = StableGUIHelper()
 // contents = PatientBoxPanel(monitorsPane,s)
}

case class StableGUIHelper()     extends BoxPanel(Orientation.Horizontal) {
  val b1 = new Button {
    text = "Update"
  }

  val SymptomaticDespiteOMT = new CheckBox("Symptomatic despite OMT")
  val AnatomicalPatterns = new CheckBox("Anatomical patterns")
  val SignLmStenois = new CheckBox(" Sign. LM-Stenosis ")
  val SignLADDisease = new CheckBox(" Sign. LAD Disease ")
  val SeverePatternofCAD = new CheckBox("Severe Patterns of CAD")
  val NonProximalLAD = new CheckBox("Non-Proximal LAD")
  val SeverePatternsOfCAD = new CheckBox("Severe Patterns Of CAD")


  contents ++= List(
    new BoxPanel(Orientation.Vertical) {
      contents += SymptomaticDespiteOMT
      contents += AnatomicalPatterns
      contents += SignLmStenois
      contents += SignLADDisease
      contents += SeverePatternofCAD
      contents += NonProximalLAD
      contents += SeverePatternsOfCAD
      contents += b1
    }
  )
  reactions += {
    case event.ButtonClicked(`b1`) => {}
    case SymptomaticDespiteOMT =>
    case AnatomicalPatterns =>
    case SignLmStenois =>
    case SignLADDisease =>
    case SeverePatternofCAD =>
    case NonProximalLAD =>

  }
  listenTo(b1, SymptomaticDespiteOMT, AnatomicalPatterns, SignLmStenois, SignLADDisease, SeverePatternofCAD, NonProximalLAD, SeverePatternsOfCAD)
}


class StemiGUI(s: String)     extends Frame {

  val checkboxes = List("Check", "Check2", "Check3")
  var checks = checkboxes map ((s: String) => new CheckBox(s))

  contents = new BoxPanel(Orientation.Vertical) {

    contents ++= checks

  }
}


class NonStemiGUI(s: String)
  extends Frame {

  val checkboxes = List("Check", "Check2", "Check3")
  var checks = checkboxes map ((s: String) => new CheckBox(s))

  contents = new BoxPanel(Orientation.Vertical) {

    contents ++= checks
  }
}


*/
